<?php

/**
 * SL_Admin class
 *
 * Handles all the admin aspects
 *
 */

class SL_Admin
{
    private $plugin_name;
    private $version;

    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    public function sl_additional_cron_intervals()
    {
        $interval['sl_remove_empty_carts_interval'] = [ //Defining cron Interval for removing abandoned carts that do not have products
            'interval' => 12 * 60 * 60,
            'display' => 'Twice a day'
        ];
        $interval['sl_run_sendlaneapi_interval'] = [
            'interval' => 10 * 60,
            'display' => 'Every 10 mins'
        ];
        return $interval;
    }

    /**
     * count of abandoned carts
     */
    public function abandoned_cart_count()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix
        $total_items = $wpdb->get_var("SELECT COUNT(id) FROM $table_name");

        return $total_items;
    }

    /**
     * Register plugin text domain
     */
    function sl_text_domain()
    {
        return load_plugin_textdomain(SL_TX, false, basename(plugin_dir_path(__DIR__)) . '/languages');
    }

    /**
     * Delete saved carts with no items
     */
    function delete_empty_carts()
    {

        global $wpdb;
        $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix

        //Deleting row from database
        $count = $wpdb->query(
            $wpdb->prepare(
                "DELETE FROM " . $table_name . "
				WHERE cart_contents = '' AND
				time < (NOW() - INTERVAL %d DAY)",
                1
            )
        );

        if ($count) {
            $public = new SL_Public(SL_SLUG, SL_VERSION);
            $public->sldecrease_captured_abandoned_cart_count($count);
        }
    }

    /**
     * Admin styles & scripts enqueue
     */
    public function sl_enqueue_styles()
    {

        $screen = get_current_screen();

        if (!is_object($screen)) {
            return;
        }

        if ($screen->id == 'woocommerce_page_wc-settings') {
            wp_enqueue_style($this->plugin_name, SL_URL . 'admin/css/sl-admin.css', false, $this->version, 'all');
        }
    }

    public function sl_enqueue_scripts()
    {
        $screen = get_current_screen();

        if (!is_object($screen) || $screen->id != 'woocommerce_page_wc-settings') {
            return;
        }

        wp_enqueue_script($this->plugin_name, SL_URL . 'admin/js/sl-admin.js', ['jquery'], $this->version, false);
        wp_localize_script($this->plugin_name, 'ajaxLink', ['ajaxurl' => admin_url('admin-ajax.php')]);
    }

    /**
     *
     * Check each row of table and send appropriate rows to sendlane api
     *
     */
    function execute_sendlane_api()
    {
        if (!get_option('sendlane_domain')) {
            return;
        }

        global $wpdb;
        $table_name = $wpdb->prefix . SL_TABLE;
        $webhooks_table = $wpdb->prefix . WC_WEBHOOKS;
        $webhook_topic = SL_WEBHOOK_TOPIC;
        $webhook = [];

        $webhooks = $wpdb->get_results("SELECT * FROM {$webhooks_table} WHERE `topic` = '{$webhook_topic}'", ARRAY_A);

        foreach ($webhooks as $w) {
            $webhook = $w;
            break;
        }

        $rows = $wpdb->get_results("SELECT * FROM {$table_name} WHERE cart_contents != '' AND sendlane_response IS NULL AND `time` <= NOW() - INTERVAL 5 MINUTE", ARRAY_A);

        if (!$webhook || !count($rows)) {
            return;
        }

        require SL_PATH . 'webhook/sl-webhook.php';
        $connector = new SL_Webhook();

        foreach ($rows as $row) {
            if (!$row['email']) {
                continue;
            }

            $sl_session = $row['session_id'];

            $connector->sl_set_abandoned_checkout_webhook($webhook, $sl_session);

            $billing = $shipping = $other = [];
            $otherFields = unserialize($row['other_fields']);

            foreach ($otherFields as $key => $otherField) {
                if (stripos($key, 'billing_') !== false) {
                    $billing[str_replace('sl_billing_', '', $key)] = $otherField;
                } elseif (stripos($key, 'shipping_')) {
                    $shipping[str_replace('sl_shipping_', '', $key)] = $otherField;
                } else {
                    $other[str_replace('sl_', '', $key)] = $otherField;
                }
            }

            $other['time'] = $row['time'];

            $data = array_merge([
                'id' => $sl_session,
                'status' => 'Pending',
                'email' => $row['email'],
                'first_name' => $row['name'],
                'last_name' => $row['surname'],
                'phone' => $row['phone'],
                'location' => $row['location'],
                'total' => $row['cart_total'],
                'currency' => $row['currency'],
                'cart_link' => site_url() . '/cart/?sl_session=' . $sl_session,
                'line_items' => unserialize($row['cart_contents']),
                'billing' => $billing,
                'shipping' => $shipping,
            ], $other);

            $connector->sl_curl_execute($data);

            if (stripos($connector->response, 'error') !== false) {
                $e_log = fopen(SL_PATH . 'api_error.log', 'a');
                $time = date('Y-m-d H:i:s');
                $err = "[$time] $sl_session  - $connector->response \r\n";
                fwrite($e_log, $err);
                fclose($e_log);
            } else {
                $wpdb->update(
                    $table_name,
                    [
                        'sendlane_response' => 'success',  // string
                    ],
                    ['session_id' => $sl_session],
                    [
                        '%s',
                    ],
                    ['%s']
                );
            }
        }
    }

    /**
     * print all logged errors on screen IF any
     */
    public function show_api_errors()
    {
        if (isset($_GET['sendlaneerrors'])) {
            if (!file_exists(SL_PATH . 'api_error.log')) {
                die('Hurray! There are no errors so far as everything went smooth with Sendlane API');
            }

            $read = file_get_contents(SL_PATH . 'api_error.log');
            echo $read;
            die();
        }
    }

    /**
     * Register new settings tab under woocommerce setting tabs   *
     */
    function sl_register_settings_tab($wtabs)
    {
        $tabs = [];
        $tabs['sendlane_settings'] = __('Integration', SL_TX);
        $backup = $wtabs;

        $newtabs = array_merge(array_splice($wtabs, 0, count($wtabs) - 1), $tabs, array_splice($backup, -1));

        return $newtabs;
    }

    /***
     * Settings section for sendlane
     */
    function sl_sendlane_settings_section()
    {
        $section = '<div id="sendlane-connector">
    <style>p.submit{display:none;}</style>';
        $section .= '<h2>' . __('Sendlane Abandoned Cart for WooCommerce', SL_TX) . '</h2>';

        $sendlane_domain = get_option('sendlane_domain');

        $section .= '<p>' . __('Please specify your Abandoned Cart settings.', SL_TX) . '</p>';

        $section .= '<div id="sendlane-error">  ' . __('Could not save your settings! please try again', SL_TX) . ' </div>';

        $section .= '<table class="form-table"><tbody>
      <tr valign="top">
        <th scope="row" class="titledesc">
          <label for="sendlane_domain">' . __('Subdomain',
                SL_TX) . ' <span class="woocommerce-help-tip" data-tip="' . __("Please enter your Sendlane subdomain. It can be found in your Sendlane account settings", SL_TX) . '"></span></label>
        </th>
        <td class="forminp forminp-text">
           <input type="text" id="sendlane_domain" name="sendlane_domain" value="' . esc_attr($sendlane_domain) . '">
        </td>
      </tr>
      </tbody></table>
      <div id="sl-saving">
        <p><img class="rotateme saving" src="' . SL_URL . 'admin/assets/loading-pink.png"/> ' . __('Saving your settings', SL_TX) . '</p>
      </div>
      <div class="sendlane-connection">
      <button type="button" id="sendlane-savechanges">
      ' . __('Save Changes', SL_TX) . '</button>
      </div>
      <div id="sendlane-connection">
      <button type="button" id="sendlane-disconnect">
      <span class="connecting"><img class="rotateme" src="' . SL_URL . 'admin/assets/loading-white.png"/></span>' . __('Disconnect', SL_TX) . '</button>
      </div>
      
      </div>';

        $section .= '<input type="hidden" id="sendlane_nc" value="' . wp_create_nonce('savesendlane') . '">';

        echo $section;
    }

    /**
     * Sendlane disconnect connection
     */
    function sl_disconnect_sendlane()
    {
        if (!wp_verify_nonce($_POST['nc'], 'savesendlane')) {
            die("Unauthorized request");
        }

        delete_option('sendlane_domain');

        wp_clear_scheduled_hook('sl_run_sendlaneapi_hook');

        exit();
    }

    /**
     * save admin settings via ajax
     */
    function sl_save_admin_settings()
    {
        if (!wp_verify_nonce($_POST['nc'], 'savesendlane')) {
            die("Unauthorized request");
        }

        if (!wp_next_scheduled('sl_remove_empty_carts_hook')) {
            wp_schedule_event(time(), 'sl_remove_empty_carts_interval', 'sl_remove_empty_carts_hook');
        }

        if (!wp_next_scheduled('sl_run_sendlaneapi_hook')) {
            wp_schedule_event(time(), 'sl_run_sendlaneapi_interval', 'sl_run_sendlaneapi_hook');
        }

        update_option('sendlane_domain', $_POST['domain']);

        exit();
    }

    /**
     * custom action link on plugins page
     */
    function sl_plugin_action_links($actions, $plugin_file)
    {
        if (!is_array($actions)) {
            return $actions;
        }

        $action_links = [];
        $action_links['sl_settings'] = [
            'label' => __('Settings', SL_TX),
            'url' => admin_url('/admin.php?page=wc-settings&tab=sendlane_settings')
        ];

        return $this->sl_display_plugin_action_links($actions, $plugin_file, $action_links, 'before');
    }

    function sl_display_plugin_action_links($actions, $plugin_file, $action_links = [], $position = 'after')
    {
        static $plugin;
        if (!isset($plugin)) {
            $plugin = SL_BASENAME;
        }
        if ($plugin === $plugin_file && !empty($action_links)) {
            foreach ($action_links as $key => $value) {
                $link = [$key => '<a href="' . $value['url'] . '">' . $value['label'] . '</a>'];
                if ('after' === $position) {
                    $actions = array_merge($actions, $link);
                } else {
                    $actions = array_merge($link, $actions);
                }
            }
        }
        return $actions;
    }
}
